;*** RFM12BP_DataResponse.asm ***/
;------------------------------------------------
; Author: Torsten Knorr
; E-mail: create-soft@freenet.de
; copyright(C) 2016 by Torsten Knorr
; This library is free software, without any warranty and restricted to private use.
; You cannot use this software for commercial gains without my permission.
; If you are building software and releasing it based on this,
; you should retain this message in your source and cite it's origin.
; Otherwise just take whatever code snipplets you need.
;*** BUGS ***;
; Maybe you'll find some. Please let me know.
; By the way I am pleased with every kind of feedback.
; Improvements, comments and wishes are welcome.
;------------------------------------------------

 .equ __RFM12BP_RESPONSE__ = 1

 .equ FREQ_869_525 = 0x0778

;------------------------------------------------

 .nolist
 .include "RFM12BP_Lib/m48def.inc"
 .include "RFM12BP_Lib/RFM12BPCmd.h"
 .include "RFM12BP_Lib/RFM12BP.inc"
 .include "RFM12BP_Lib/transmit.inc"
 .include "RFM12BP_Lib/receive.inc"
 .include "RFM12BP_Lib/vdi.inc"
 .include "RFM12BP_Lib/SPI.inc"
 .include "RFM12BP_Lib/TWI.inc"
 .include "RFM12BP_Lib/timer.inc"
 .include "RFM12BP_Lib/LED.inc"
 .include "RFM12BP_Lib/cmd_exe.inc"
 .list

;------------------------------------------------

 .dseg
 .org SRAM_START
 s_tx_buff:
 s_preamble:    .byte BYTES_PREAMBLE
 s_frame_start: .byte BYTES_FR_START
 s_identifier:  .byte BYTES_ID
 s_time:        .byte BYTES_TIME
 s_cmd:         .byte BYTES_CMD
 s_para_1:      .byte 1
 s_para_2:      .byte 1
 s_data:        .byte BYTES_DATA
 s_crc:         .byte BYTES_CRC
 s_finale:      .byte BYTES_FINALE
 s_tx_buff_end:

;------------------------------------------------

 .cseg
 .org 0x00

    rjmp RESET            ; Reset Handler                      0x0000
    rjmp EXT_INT0         ; IRQ0 Handler                       0x0001
    rjmp EXT_INT1         ; IRQ1 Handler                       0x0002
    reti; rjmp PCINT0     ; PCINT0 Handler                     0x0003
    reti; rjmp PCINT1     ; PCINT1 Handler                     0x0004
    rjmp PCINT2_20_VDI    ; PCINT2 Handler                     0x0005
    reti; rjmp WDT        ; Watchdog Timer Handler             0x0006
    reti; rjmp TIM2_COMPA ; Timer2 Compare A Handler           0x0007
    reti; rjmp TIM2_COMPB ; Timer2 Compare B Handler           0x0008
    rjmp TIM2_OVF         ; Timer2 Overflow Handler            0x0009
    reti; rjmp TIM1_CAPT  ; Timer1 Capture Handler             0x000A
    rjmp TIM1_COMPA       ; Timer1 Compare A Handler           0x000B
    reti; rjmp TIM1_COMPB ; Timer1 Compare B Handler           0x000C
    reti; rjmp TIM1_OVF   ; Timer1 Overflow Handler            0x000D
    reti; rjmp TIM0_COMPA ; Timer0 Compare A Handler           0x000E
    reti; rjmp TIM0_COMPB ; Timer0 Compare B Handler           0x000F
    reti; rjmp TIM0_OVF   ; Timer0 Overflow Handler            0x0010
    reti; rjmp SPI_STC    ; SPI Transfer Complete Handler      0x0011
    reti; rjmp USART_RXC  ; USART, RX Complete Handler         0x0012
    reti; rjmp USART_UDRE ; USART, UDR Empty Handler           0x0013
    reti; rjmp USART_TXC  ; USART, TX Complete Handler         0x0014
    reti; rjmp ADC        ; ADC Conversion Complete Handler    0x0015
    reti; rjmp EE_RDY     ; EEPROM Ready Handler               0x0016
    reti; rjmp ANA_COMP   ; Analog Comparator Handler          0x0017
    rjmp TWI              ; 2-wire Serial Interface Handler    0x0018
    reti; rjmp SPM_RDY    ; Store Program Memory Ready Handler 0x0019

;------------------------------------------------

 EXT_INT0:
    M_RFM12BP_INT0_TRANSMIT

 EXT_INT1:
    M_RFM12BP_INT1_RECEIVE

 PCINT2_20_VDI:
    M_RFM12BP_PCINT2_20_VDI

 TIM1_COMPA:
    M_RFM12BP_TIM1_INT

 TIM2_OVF:
    M_RFM12BP_TIM2_INT

 TWI:
    M_RFM12BP_TWI_INT_RESPONSE

;------------------------------------------------

 RESET:

    M_PROGRAM_START
    M_INIT_WATCHDOG
    M_INIT_PORTS
    M_INIT_SPI
    M_INIT_TWI s_data_end
    M_INIT_INT0_TRANSMIT
    M_INIT_INT1_RECEIVE
    M_INIT_PCINT2_20_VDI
    M_INIT_TIMER_1
    M_INIT_TIMER_2
    M_INIT_SRAM TX_BEGIN_SIZE, s_preamble, e_preamble
    M_INIT_SRAM BYTES_FINALE, s_finale, e_finale

 RFM12BP_START:

    rcall RFM12BP_INIT

    M_LED_LISTEN_ON
    M_RFM12BP_LISTEN RX_REQUEST_SIZE, s_identifier

;------------------------------------------------
;*** Main program start ***;

 MAIN_LOOP:
    wdr

    sbic STATUS_REG,    NEW_INIT
    rjmp RFM12BP_START

    TOGGLE_PORT_BIT PORTD, LED_RUN, TIME_1S

    sbis STATUS_REG,    VDI_STAT_OK
    rjmp MAIN_LOOP

;------------------------------------------------

    M_STOP_TIMER_1
    M_LED_RUN_OFF

    M_LED_LISTEN_OFF
    M_LED_RX_ON
    M_RFM12BP_RECEIVE RX_REQUEST_SIZE, s_identifier
    M_LED_RX_OFF

    M_RFM12BP_CHECK_ID

    M_RFM12BP_CHECK_CMD

    M_LED_TX_ON
    M_RFM12BP_TRANSMIT TX_RESPONSE_SIZE, s_tx_buff
    M_LED_TX_OFF

 NO_ACTION:
    M_LED_LISTEN_ON
    M_RFM12BP_LISTEN RX_REQUEST_SIZE, s_identifier

    rjmp MAIN_LOOP

;------------------------------------------------

 RFM12BP_INIT:
    M_RFM12BP_INIT

;------------------------------------------------

 RFM12BP_CMD_EXE_TABLE:
    M_RFM12BP_CMD_EXE_TABLE

;------------------------------------------------

 EEPROM_READ_BYTE:
    M_EEPROM_READ_BYTE

;------------------------------------------------

 EEPROM_WRITE_BYTE:
    M_EEPROM_WRITE_BYTE

;------------------------------------------------

