;*** timer.inc ***;
;------------------------------------------------
; Author: Torsten Knorr
;------------------------------------------------

 .ifndef __RFM12BP_TIMER__
 .equ    __RFM12BP_TIMER__ = 1

 .include "RFM12BP.inc"

;------------------------------------------------

 .macro M_INIT_TIMER_1

    ldi  R_TEMP_L,      (1 << OCF1A)
    out  TIFR1,         R_TEMP_L

    ldi  R_TEMP_L,      ((1 << WGM12) | (1 << CS12) | (1 << CS10))
    sts  TCCR1B,        R_TEMP_L

    clr  R_TEMP_L
    sts  TIMSK1,        R_TEMP_L

    sts  TCCR1A,        R_TEMP_L
    sts  TCCR1C,        R_TEMP_L

    sts  OCR1AH,        R_TEMP_L
    sts  OCR1AL,        R_TEMP_L

    sts  TCNT1H,        R_TEMP_L
    sts  TCNT1L,        R_TEMP_L

    cbi  STATUS_REG,    TIMER_1_RUN

 .endmacro

;------------------------------------------------

 .macro M_INIT_TIMER_2

    ldi  R_TEMP_L,      (1 << TOV2)
    out  TIFR2,         R_TEMP_L

    ldi  R_TEMP_L,      ((1 << CS22) | (1 << CS21) | (1 << CS20))
    sts  TCCR2B,        R_TEMP_L

    clr  R_TEMP_L
    sts  TIMSK2,        R_TEMP_L

    sts  TCCR2A,        R_TEMP_L

    sts  OCR2A,         R_TEMP_L

    sts  TCNT2,         R_TEMP_L

    cbi  STATUS_REG,    TIMER_2_RUN
    cbi  STATUS_REG,    NEW_INIT

 .endmacro

;------------------------------------------------

 .macro M_STOP_TIMER_1

    clr  R_TEMP_L
    sts  TIMSK1,        R_TEMP_L

    cbi  STATUS_REG,    TIMER_1_RUN

 .endmacro

;------------------------------------------------

 .macro M_STOP_TIMER_2

    clr  R_TEMP_L
    sts  TIMSK2,        R_TEMP_L

    cbi  STATUS_REG,    TIMER_2_RUN

 .endmacro

;------------------------------------------------

 .macro M_RFM12BP_TIM1_INT

    in   RI_SREG,       SREG

    clr  RI_TEMP_L
    sts  TIMSK1,        RI_TEMP_L

    cbi  STATUS_REG,    TIMER_1_RUN

    out  SREG,          RI_SREG
    reti

 .endmacro

;------------------------------------------------

 .macro M_RFM12BP_TIM2_INT

    in   RI_SREG,       SREG

    dec  R_T2_COUNT
    brne TIMER_2_RUNNING

    clr  RI_TEMP_L
    sts  TIMSK2,        RI_TEMP_L

    sbi  STATUS_REG,    NEW_INIT
    cbi  STATUS_REG,    TIMER_2_RUN

 TIMER_2_RUNNING:

    out  SREG,          RI_SREG
    reti

 .endmacro

;------------------------------------------------

 .equ TIME_1MS    = 0x0009
 .equ TIME_2MS    = 0x0013
 .equ TIME_6MS    = 0x003A
 .equ TIME_10MS   = 0x0061
 .equ TIME_200MS  = 0x07A1
 .equ TIME_1S     = 0x2625
 .equ TIME_6711MS = 0xFFFF

 .macro M_SET_TIMER_1

    ldi  R_TEMP_L,      low(@0)
    ldi  R_TEMP_H,      high(@0)

; disable global interrupts
    cli

    sts  OCR1AH,        R_TEMP_H
    sts  OCR1AL,        R_TEMP_L

    clr  R_TEMP_L
    sts  TCNT1H,        R_TEMP_L
    sts  TCNT1L,        R_TEMP_L

    ldi  R_TEMP_L,      (1 << OCF1A)
    out  TIFR1,         R_TEMP_L

    ldi  R_TEMP_L,      (1 << OCIE1A)
    sts  TIMSK1,        R_TEMP_L

    sbi  STATUS_REG,    TIMER_1_RUN

; enable global interrupts
    sei

 .endmacro

;------------------------------------------------
; 1 / 10_000_000 = 0.000_000_1 * 1024 prescaler * 256 = 0.02621s = 26ms

 .macro M_SET_TIMER_2

; disable global interrupts
    cli

    ldi  R_TEMP_L,      @0
    mov  R_T2_COUNT,    R_TEMP_L

    clr  R_TEMP_L
    sts  TCNT2,         R_TEMP_L

    ldi  R_TEMP_L,      (1 << TOV2)
    out  TIFR2,         R_TEMP_L

    ldi  R_TEMP_L,      (1 << TOIE2)
    sts  TIMSK2,        R_TEMP_L

    sbi  STATUS_REG,    TIMER_2_RUN
    cbi  STATUS_REG,    NEW_INIT

; enable global interrupts
    sei

 .endmacro

;------------------------------------------------

 .equ TIME_1_BYTE = 0x0014

 .macro M_SET_RX_TIMEOUT

    ldi  R_TEMP_L,      low(@0)
    ldi  R_TEMP_H,      TIME_1_BYTE
    mul  R_TEMP_L,      R_TEMP_H
    movw R_TEMP_L,      R_MUL_L

; disable global interrupts
    cli

    sts  OCR1AH,        R_TEMP_H
    sts  OCR1AL,        R_TEMP_L

    clr  R_TEMP_L
    sts  TCNT1H,        R_TEMP_L
    sts  TCNT1L,        R_TEMP_L

    ldi  R_TEMP_L,      (1 << OCF1A)
    out  TIFR1,         R_TEMP_L

    ldi  R_TEMP_L,      (1 << OCIE1A)
    sts  TIMSK1,        R_TEMP_L

    sbi  STATUS_REG,    TIMER_1_RUN

; enable global interrupts
    sei

 .endmacro

;------------------------------------------------

 .macro M_WAIT_TIMER_1

 WAIT_TIMER_1_LOOP:
    wdr
    sbic STATUS_REG,    TIMER_1_RUN
    rjmp WAIT_TIMER_1_LOOP

 .endmacro

;------------------------------------------------

 .macro M_WAIT_TIMER_2

 WAIT_TIMER_2_LOOP:
    wdr
    sbic STATUS_REG,     TIMER_2_RUN
    rjmp WAIT_TIMER_2_LOOP

 .endmacro

;------------------------------------------------

 .macro M_CHECK_TIMER_1

    sbis STATUS_REG,    TIMER_1_RUN

 .endmacro

;------------------------------------------------

 .macro M_CHECK_TIMER_2

    sbis STATUS_REG,    TIMER_2_RUN

 .endmacro

;------------------------------------------------

 .macro TOGGLE_PORT_BIT

    sbic STATUS_REG,    TIMER_1_RUN
    rjmp TOGGLE_NOT_BIT

    in   R_TEMP_L,      @0
    ldi  R_TEMP_H,      (1 << @1)
    eor  R_TEMP_L,      R_TEMP_H
    out  @0,            R_TEMP_L

    M_SET_TIMER_1 @2

    TOGGLE_NOT_BIT:

 .endmacro

;------------------------------------------------

 .endif

;------------------------------------------------
